@@ -1,3 +1,5 @@
+import logging
+
 from SeleniumLibrary import SeleniumLibrary
 from SeleniumLibrary.keywords import BrowserManagementKeywords, ScreenshotKeywords, WaitingKeywords, \
                                      FormElementKeywords, ElementKeywords, JavaScriptKeywords
@@ -9,11 +11,10 @@ import docker
 import os
 import socket
 import time
-import shutil
 from datetime import datetime
 
 from ftsa.core.properties import VERSION, REMOTE_VERSION, SLEEP_SECONDS, REC_VIDEO_VERSION
-from ftsa.core.utils import extract_name_from, extract_type_from, sanitize_camel_case
+from ftsa.core.utils import extract_name_from, extract_type_from, sanitize_camel_case, get_project_properties
 
 
 @library(scope='GLOBAL', version=VERSION)
@@ -23,13 +24,13 @@ class FTSASeleniumLibrary(SeleniumLibrary):
         SeleniumLibrary.__init__(self)
 
     @keyword
-    def abrir_o_navegador(self, url, browser, maximized=True, speed="0.3s", timeout="20s", implicit_wait="10s"):
-        self.open_browser(self, url, browser, maximized=maximized, speed=speed, timeout=timeout,
-                          implicit_wait=implicit_wait)
-
-    @keyword
-    def open_browser(self, url, browser="chrome", remote_url=None, maximized=True, speed="0.3s", timeout="20s",
-                     implicit_wait="10s", download_path=f"output{os.sep}downloads"):
+    def open_browser(self, url=get_project_properties().get('prop', 'URL'),
+                     browser=get_project_properties().get('prop', 'BROWSER'), remote_url=None,
+                     maximized=True, speed=get_project_properties().get('prop', 'SPEED'),
+                     timeout=get_project_properties().get('prop', 'TIMEOUT'),
+                     implicit_wait=get_project_properties().get('prop', 'IMPLICIT_WAIT'),
+                     idiom=get_project_properties().get("prop", "IDIOM"),
+                     download_path=f"output{os.sep}downloads"):
         """*FTSA opens a new browser instance*
 
         Open the ``browser`` with initial tests ``url``.
@@ -41,6 +42,7 @@ class FTSASeleniumLibrary(SeleniumLibrary):
         | speed         | 0.3s   |
         | timeout       | 20s    |
         | implicit_wait | 10s    |
+        | idiom         | en-US  |
         | download_path | output/downloads (Chrome, only) |
 
         _Returns_ the *page title*
@@ -53,13 +55,19 @@ class FTSASeleniumLibrary(SeleniumLibrary):
             logger.info(f'Download directory created: "{download_path}"')
         else:
             logger.info(f'Directory already exists: "{download_path}"')
+
         # Configuring BROWSER OPTIONS
-        config_options = None
         browser_management = BrowserManagementKeywords(self)
         if browser.lower() == 'firefox':
             from selenium.webdriver.firefox.options import Options, FirefoxProfile
             config_options = Options()
             profile = FirefoxProfile(FirefoxProfile.DEFAULT_PREFERENCES)
+            profile.set_preference("browser.download.folderList", 2)
+            profile.set_preference("browser.download.manager.showWhenStarting", False)
+            profile.set_preference("browser.download.dir", download_path)
+            profile.set_preference("browser.helperApps.alwaysAsk.force", False)
+            profile.set_preference("intl.accept_languages", idiom)
+            profile.update_preferences()
             config_options.profile = profile
             config_options.binary = which("firefox")
             logger.info(f'Browser FIREFOX config options enabled.')
@@ -81,10 +89,12 @@ class FTSASeleniumLibrary(SeleniumLibrary):
                                                              "download.prompt_for_download": False,
                                                              "download.directory_upgrade": True,
                                                              "safebrowsing.enabled": True})
+            config_options.add_argument(f'--lang={idiom}')
             config_options.binary = which("chrome")
             logger.info(f'Browser CHROME config options enabled.')
         else:
             raise RuntimeError(f'{browser} browser is not supported.')
+
         # Configuring MAXIMIZED
         if maximized:
             logger.info(f'Setting {browser.upper()} browser to open MAXIMIZED.')
@@ -96,7 +106,7 @@ class FTSASeleniumLibrary(SeleniumLibrary):
         logger.info(f'Setting SELENIUM IMPLICIT WAIT to "{implicit_wait}".')
         browser_management.set_selenium_implicit_wait(implicit_wait)
         # Open Browsing with/without remote URL
-        if remote_url is not None:
+        if remote_url is not None and os.getenv('NO_DOCKER_EXECUTION') != 'nodocker':
             browser_management.open_browser(url, browser, options=config_options, remote_url=remote_url)
             logger.info(f'Browser {browser.upper()} opened in remote url {remote_url}.')
         else:
@@ -117,44 +127,49 @@ class FTSASeleniumLibrary(SeleniumLibrary):
         browser_management.close_all_browsers()
 
     @keyword
-    def login_user(self, username, password):
+    def login_user(self, username, password, url=get_project_properties().get('prop', 'URL'), cas2=True):
         """*FTSA Login the user*
 
-        FTSA Login the user with ``username`` and ``password``.
+        FTSA Login the user with ``username`` and ``password`` into CAS2, before redirecting to the ``url``.
 
         | *Obs:* | To properly work, page must contains id locators to ``id:username`` and ``id:password``; and a submit button with ``name:submit`` locator |
 
         """
-        logger.info(f'FTSA logins the "{username}" user with password "{password}"')
+        logger.info(f'FTSA logins the "{username}" user with password "{password}" and, if cas2=True, redirects to "{url}"')
+        browser_management = BrowserManagementKeywords(self)
         form_elements = FormElementKeywords(self)
         elements = ElementKeywords(self)
         waiting = WaitingKeywords(self)
+        project = get_project_properties()
+
+        cas_url = project.get('prop', 'CAS_URL')
+        if cas2:
+            # Go to CAS and Login
+            logger.info(f'CAS2 Detected. Going to "{cas_url}"')
+            browser_management.go_to(cas_url)
+
+            # Click Login Again button
+            login_again_button = "xpath://div[@id='content']//a[@href='/cas2/login']"
+            waiting.wait_until_element_is_visible(locator=login_again_button)
+            elements.click_element(locator=login_again_button)
+        elif get_project_properties().get('prop', 'URL') != url:
+            # Go to keyword specified URL
+            logger.info(f'Going to keyword specified URL "{url}"')
+            browser_management.go_to(url)
+
+        # Set Username, Password and Submit
         waiting.wait_until_element_is_visible(locator='id:username')
         waiting.wait_until_element_is_visible(locator='id:password')
         form_elements.input_text(locator='id:username', text=username, clear=True)
         form_elements.input_text(locator='id:password', text=password, clear=True)
-        waiting.wait_until_element_is_enabled(locator='name:submit')
-        elements.click_element('name:submit')
 
-    @keyword
-    def login_user_with_redirect(self, username, password, url=None):
-        """*FTSA Login the user*
+        elements.click_element("xpath://*[@type='submit']")
 
-        FTSA Login the user with ``username`` and ``password``, redirecting to the ``url`` at the end.
-
-        | *Obs:* | To properly work, page must contains id locators to ``id:username`` and ``id:password``; and a submit button with ``name:submit`` locator |
-
-        """
-        browser_management = BrowserManagementKeywords(self)
-        elements = ElementKeywords(self)
-        waiting = WaitingKeywords(self)
-        if url is not None:
+        # Wait 3 seconds and go to system URL
+        time.sleep(3)
+        if cas2 and url != cas_url:
+            logger.info(f'CAS2 Detected and "{url}" is not equals to "{cas_url}. Going to "{url}"')
             browser_management.go_to(url)
-        waiting.wait_until_element_is_visible(locator='xpath://*[@id="content"]/div/div/p[3]/a')
-        elements.click_element(locator='xpath://*[@id="content"]/div/div/p[3]/a')
-        self.login_user(username, password)
-        waiting.assert_page_contains(locator='//*[@id="content"]/div[@class="alert alert-success"]')
-        browser_management.go_to(url)
 
     @keyword
     def input_text_js(self, locator, value):
@@ -233,15 +248,15 @@ class FTSASeleniumLibrary(SeleniumLibrary):
             raise Exception(f'"{the_type}" is not a valid locator type!')
 
     @keyword
-    def init_remote_server(self, browser="chrome",
-                           grid_name="grid", selenium_name='selenium',
+    def init_remote_server(self, browser=get_project_properties().get('prop', 'BROWSER'),
+                           network_name="grid", selenium_name='selenium',
                            port="4444", ):
         """*FTSA initialize remote server*
 
         FTSA creates a new container instance to run FTSA-TJPB tests inside, with the following configurations:
 
         | browser | The name of the browser: chrome, firefox, opera or edge are allowed |
-        | grid_name | The name used to run/up network, by default: grid |
+        | network_name | The name used to run/up network, by default: grid |
         | selenium_name | The name used to run/up selenium container, by default: selenium |
         | port | The external available port (entrypoint) to the container, by default: 4444 |
 
@@ -250,146 +265,182 @@ class FTSASeleniumLibrary(SeleniumLibrary):
         ***If SELENIUM_EXEC_HOST environment variable is set, it will be used to replace the default hostname machine
         (you may use IP directly, for example).***
         """
-        docker_client = docker.from_env()
-        date_str = datetime.now().strftime("%Y%m%d%H%M%S")
-        if browser == "firefox":
-            browser_image_name = f'selenium/standalone-firefox:{REMOTE_VERSION}'
-        elif browser == "opera":
-            browser_image_name = f'selenium/standalone-opera:{REMOTE_VERSION}'
-        elif browser == "edge":
-            browser_image_name = f'selenium/standalone-edge:{REMOTE_VERSION}'
-        elif browser == "chrome" or browser == "chromium":
-            browser_image_name = f'selenium/standalone-chrome:{REMOTE_VERSION}'
+        # Does execution allows docker?
+        if os.getenv('NO_DOCKER_EXECUTION') == 'nodocker':
+            logger.info('Keyword ignored. Docker is not allowed for this execution')
+            return {
+                "host": None,
+                "port": None,
+                "network_name": None,
+                "selenium_name": None
+            }
         else:
-            raise RuntimeError(f'{browser} browser is not supported.')
+            docker_client = docker.from_env()
+            if browser == "firefox":
+                browser_image_name = f'selenium/standalone-firefox:{REMOTE_VERSION}'
+            elif browser == "opera":
+                browser_image_name = f'selenium/standalone-opera:{REMOTE_VERSION}'
+            elif browser == "edge":
+                browser_image_name = f'selenium/standalone-edge:{REMOTE_VERSION}'
+            elif browser == "chrome" or browser == "chromium":
+                browser_image_name = f'selenium/standalone-chrome:{REMOTE_VERSION}'
+            else:
+                raise RuntimeError(f'{browser} browser is not supported.')
 
-        grid_name_env = os.getenv('USER_GRID_NAME')
-        if grid_name_env is None:
-            grid_name = f'{date_str}_{grid_name}'
-            network = docker_client.networks.create(grid_name)
+            # Initializing NETWORK
+            network_name_env = os.getenv('USER_NETWORK_NAME')
+            if network_name_env is not None:
+                network_name = network_name_env
+            try:
+                network = docker_client.networks.get(network_name)
+            except BaseException:
+                network = docker_client.networks.create(network_name)
             logger.info(f'{network.attrs}')
-            logger.info(f'Network {grid_name} initialized.')
-        else:
-            grid_name = grid_name_env
+            logger.info(f'Network "{network_name}" initialized.')
 
-        selenium_name_env = os.getenv('SELENIUM_NAME')
-        if selenium_name_env is None:
-            selenium_name = f'{date_str}_{selenium_name}'
-        else:
-            selenium_name = selenium_name_env
-
-        try:
-            selenium = docker_client.containers.get(selenium_name)
-        except BaseException:
-            selenium = docker_client.containers.run(
-                name=f'{selenium_name}', network=f'{grid_name}',
-                image=f'{browser_image_name}',
-                detach=True, privileged=False,
-                ports={
-                    f'{port}': '4444',
-                    '6900': '5900'
-                }
-            )
+            # Initializing SELENIUM container
+            selenium_name_env = os.getenv('SELENIUM_CONTAINER_NAME')
+            if selenium_name_env is not None:
+                selenium_name = selenium_name_env
+            try:
+                selenium = docker_client.containers.get(selenium_name)
+            except BaseException:
+                selenium = docker_client.containers.run(
+                    name=f'{selenium_name}', network=f'{network_name}',
+                    image=f'{browser_image_name}',
+                    detach=True, privileged=False,
+                    ports={
+                        f'{port}': '4444',
+                        '6900': '5900'
+                    }
+                )
+            selenium.logs()
             logger.info(f'{selenium.attrs}')
-            logger.info(f'Selenium container "{selenium_name}" initialized into "{grid_name}" network.')
-        selenium.logs()
+            logger.info(f'Selenium container "{selenium_name}" initialized into "{network_name}" network.')
+            time.sleep(SLEEP_SECONDS)
 
-        host = os.getenv('SELENIUM_EXEC_HOST')
-        if host is None:
-            # logger.info(f'SELENIUM_EXEC_HOST environment variable was NOT FOUND!')
-            # network = docker_client.networks.get(grid_name)
-            # host = network.attrs["Containers"][selenium.id]["IPv4Address"].split("/")[0]
-            # if host is None:
-            #     logger.info(f'SELENIUM_EXEC_HOST was NOT FOUND inside {grid_name} network!')
-            host = socket.gethostbyname(socket.gethostname())
-        logger.info(f'Server {host}:{port} initialized.')
-
-        time.sleep(SLEEP_SECONDS)
-        return {
-            "host": host,
-            "port": port,
-            "grid_name": grid_name,
-            "selenium_name": selenium_name
-        }
+            host = os.getenv('SELENIUM_EXEC_HOST')
+            if host is None:
+                host = socket.gethostbyname(socket.gethostname())
+            logger.info(f'Server {host}:{port} initialized.')
 
+            return {
+                "host": host,
+                "port": port,
+                "network_name": network_name,
+                "selenium_name": selenium_name
+            }
 
     @keyword
-    def end_remote_server(self, grid_name="grid", selenium_name="selenium"):
+    def end_remote_server(self, network_name="grid", selenium_name="selenium"):
         """*FTSA finalizes remote server*
 
         FTSA finalizes remote server:
 
-        | grid_name | The name used to run/up network, by default: grid |
+        | network_name | The name used to run/up network, by default: grid |
         | selenium_name | The name used to run/up selenium container, by default: selenium |
 
         ***If USER_EXEC_CONTAINER environment variable is set, it will be used to finish the server.***
         """
-        docker_client = docker.from_env()
+        # Does execution allows docker?
+        if os.getenv('NO_DOCKER_EXECUTION') == 'nodocker':
+            logger.info('Keyword ignored. Docker is not allowed for this execution')
+        else:
+            docker_client = docker.from_env()
 
-        try:
-            selenium = docker_client.containers.get(selenium_name)
-            selenium.stop()
-            selenium.remove()
-        except BaseException:
-            logger.info(f'There is no {selenium_name} container available.')
+            # Stopping and removing Selenium container
+            selenium_name_env = os.getenv('SELENIUM_CONTAINER_NAME')
+            if selenium_name_env is not None:
+                selenium_name = selenium_name_env
+            try:
+                selenium = docker_client.containers.get(selenium_name)
+                selenium.stop()
+                selenium.remove()
+                logger.info(f'Selenium container "{selenium_name}" stopped and removed.')
+            except BaseException:
+                logger.info(f'There is no {selenium_name} container available.')
+            time.sleep(SLEEP_SECONDS)
 
-        try:
-            network = docker_client.networks.get(grid_name)
-            network.remove()
-        except BaseException:
-            logger.info(f'There is no {grid_name} network available.')
-        time.sleep(SLEEP_SECONDS)
+            # Removing NETWORK
+            network_name_env = os.getenv('USER_NETWORK_NAME')
+            if network_name_env is not None:
+                network_name = network_name_env
+            try:
+                network = docker_client.networks.get(network_name)
+                network.remove()
+                logger.info(f'Network "{network_name}" removed.')
+            except BaseException:
+                logger.info(f'There is no {network_name} network available.')
 
     @keyword
-    def init_record_test_video(self, test_name=None, grid_name="grid", video_name="video",
+    def init_record_test_video(self, test_name=None, video_name="video", network_name="grid",
                                selenium_name="selenium"):
         """*FTSA initialize record test video*
 
         FTSA creates a new container instance to record test video, with the following configurations:
 
         | test_name | Inform the test case name. By defult, None. |
-        | grid_name | The name used to initialize network, by default: grid |
+        | network_name | The name used to initialize network, by default: grid |
         | video_name | The name user to run/up video recording container, by default: video |
         | selenium_name | The name user to reference selenium browser container, by default: selenium |
         """
-        docker_client = docker.from_env()
-        videos_path = f'{os.getcwd()}{os.sep}videos'
-        if not os.path.isdir(videos_path):
-            os.makedirs(videos_path)
-        if os.getenv('VOLUME_VIDEO_RECORDING') is not None:
-            videos_path = 'video_recording_volume'
+        # Does execution allows docker?
+        if os.getenv('NO_DOCKER_EXECUTION') == 'nodocker':
+            logger.info('Keyword ignored. Docker is not allowed for this execution')
+        else:
+            docker_client = docker.from_env()
 
-        date_str = datetime.now().strftime("%Y%m%d%H%M%S")
-        if test_name is not None:
-            logger.info(f'Test Name was found with value "{test_name}"')
-            test_name = f'-{sanitize_camel_case(test_name)}'
+            # Creating videos directory AND...
+            videos_path = f'{os.getcwd()}{os.sep}videos'
+            if not os.path.isdir(videos_path):
+                os.makedirs(videos_path)
+            # ... Setting volume video recording name
+            if os.getenv('VOLUME_VIDEO_RECORDING_NAME') is not None:
+                videos_path = 'video_recording_volume'
+            logger.info(f'Videos Path or Recording value enable to store recordings: "{videos_path}".')
 
-        selenium_name_env = os.getenv('SELENIUM_NAME')
-        if selenium_name_env is not None:
-            logger.info(f'Selenium Name Environment was found with value "{selenium_name_env}"')
-            selenium_name = selenium_name_env
+            # Sanitizing Test name
+            if test_name is not None:
+                test_name = f'-{sanitize_camel_case(test_name)}'
+            logger.info(f'Test Name value sanitized: "{test_name}".')
 
-        video_image_name = f'selenium/video:{REC_VIDEO_VERSION}'
-        logger.info(f'Videos path "{videos_path}" is enable to store recordings.')
-        try:
-            video = docker_client.containers.get(video_name)
-        except BaseException:
-            video = docker_client.containers.run(
-                name=f'{video_name}', network=f'{grid_name}',
-                image=f'{video_image_name}',
-                detach=True, privileged=False,
-                environment=[
-                    f'DISPLAY_CONTAINER_NAME={selenium_name}',
-                    f'FILE_NAME={date_str}{test_name}-video.mp4'
-                ],
-                volumes={
-                    f'{videos_path}': {'bind': '/videos', 'mode': 'rw'}
-                }
-            )
+            # Getting Selenium Container name
+            selenium_name_env = os.getenv('SELENIUM_CONTAINER_NAME')
+            if selenium_name_env is not None:
+                selenium_name = selenium_name_env
+            logger.info(f'Selenium Container name: "{selenium_name}".')
+
+            # Getting Network name
+            network_name_env = os.getenv('USER_NETWORK_NAME')
+            if network_name_env is not None:
+                network_name = network_name_env
+            logger.info(f'Network name: "{network_name}".')
+
+            # Initializing Videos Recorder Container
+            video_image_name = f'selenium/video:{REC_VIDEO_VERSION}'
+            video_name_env = os.getenv('VIDEOS_RECORDER_CONTAINER_NAME')
+            if video_name_env is not None:
+                video_name = video_name_env
+            try:
+                video = docker_client.containers.get(video_name)
+            except BaseException:
+                now = datetime.now().strftime("%Y%m%d%H%M%S")
+                video = docker_client.containers.run(
+                    name=f'{video_name}', network=f'{network_name}',
+                    image=f'{video_image_name}',
+                    detach=True, privileged=False,
+                    environment=[
+                        f'DISPLAY_CONTAINER_NAME={selenium_name}',
+                        f'FILE_NAME={now}{test_name}-video.mp4'
+                    ],
+                    volumes={
+                        f'{videos_path}': {'bind': '/videos', 'mode': 'rw'}
+                    }
+                )
+            video.logs()
             logger.info(f'{video.attrs}')
-            logger.info(f'Record video server initialized into "{grid_name}" network.')
-        video.logs()
-        time.sleep(SLEEP_SECONDS)
+            logger.info(f'Videos recorder container "{video_name}" initialized into "{network_name}" network.')
+            time.sleep(SLEEP_SECONDS)
 
     @keyword
     def end_record_test_video(self, video_name="video"):
@@ -399,11 +450,19 @@ class FTSASeleniumLibrary(SeleniumLibrary):
 
         | video_name | The name used to run/up video container, by default: video |
         """
-        docker_client = docker.from_env()
-        try:
-            video = docker_client.containers.get(video_name)
-            video.stop()
-            video.remove()
-        except BaseException:
-            logger.info(f'There is no {video_name} container available.')
-        time.sleep(SLEEP_SECONDS)
+        # Does execution allows docker?
+        if os.getenv('NO_DOCKER_EXECUTION') == 'nodocker':
+            logger.info('Keyword ignored. Docker is not allowed for this execution')
+        else:
+            docker_client = docker.from_env()
+            video_name_env = os.getenv('VIDEOS_RECORDER_CONTAINER_NAME')
+            if video_name_env is not None:
+                video_name = video_name_env
+            try:
+                video = docker_client.containers.get(video_name)
+                video.stop()
+                video.remove()
+                logger.info(f'Video container "{video_name}" was stopped and removed.')
+            except BaseException:
+                logger.info(f'There is no "{video_name}" container available.')
+            time.sleep(2 * SLEEP_SECONDS)
